;; Make text that comes from unimplemented tags easy to spot (and grep)
(default
  (make element gi: "FONT" 
	attributes: (list
		     (list "COLOR" "red"))))

(define ($nav-title-page-banner$)
  (literal "BANNER"))

(define ($nav-banner$)
  (literal "BANNER"))

(define ($nav-home$)
  (make element gi: "A"
	attributes: (list
		     (list "HREF" "index.htm"))
	(literal "Home")))

(define ($nav-index$)
  (make entity-ref name: "nbsp"))

(define (generate-xptr #!optional (nd (current-node)))
  (let loop ((suffix "")
	     (nd nd))
    (let ((eid (id nd)))
      (if eid
	  (string-append "I("
			 eid
			 ")"
			 (if (= (string-length suffix) 0)
			     ""
			     (string-append "C"
					    suffix)))
	  (let ((par (parent nd)))
	    (if (not (node-list-empty? par))
		(loop (string-append "("
				     (number->string (child-number nd))
				     ","
				     (gi nd)
				     ")"
				     suffix)
		      par)
		(string-append (if (= (string-length suffix) 0)
				   "R"
				   "R,C")
			       suffix)))))))

(define (book-component-html-file #!optional (nd (current-node)))
  ;; Returns "c01" for Chapter 1; "a01" for Appendix A, etc.
  ;; Assumes nd is one of PREFACE, CHAPTER, APPENDIX, REFERENCE,
  ;; GLOSSARY, BIBLIOGRAPHY, or INDEX.
  (let ((num  (two-digit-num-string (child-number nd)))
	(char (cond ((equal? (gi nd) "PREFACE") "p")
		    ((equal? (gi nd) "CHAPTER") "c")
		    ((equal? (gi nd) "APPENDIX") "a")
		    ((equal? (gi nd) "REFERENCE") "r")
		    ((equal? (gi nd) "GLOSSARY") "g")
		    ((equal? (gi nd) "BIBLIOGRAPHY") "b")
		    ((equal? (gi nd) "INDEX") "i")
		    (else "x"))))
    (string-append char num)))

(define (refentry-html-file #!optional (nd (current-node)))
  ;; Returns "c01r01" for the first RefEntry in Chapter 1, etc.
  (let ((num (two-digit-num-string (child-number nd)))
	(prefix (book-component-html-file (parent nd))))
    (string-append prefix "r" num)))

(define (sect1-html-file #!optional (nd (current-node)))
  ;; Returns "c0102" for the second Sect1 in Chapter 1, etc.
  (let ((num (two-digit-num-string (child-number nd)))
	(prefix (book-component-html-file (parent nd))))
    (string-append prefix num)))

(define (html-file #!optional (input_nd (current-node)))
  (let* ((nd (chunk-level-parent input_nd))
	 (base (cond ((equal? (gi nd) "REFENTRY") (refentry-html-file nd))
		     ((equal? (gi nd) "SECT1") (sect1-html-file nd))
		     (else (string-append (book-component-html-file 
					   nd) "01")))))
    (string-append base ".htm")))

(define chunk-level-element-list
  (list "PREFACE" "CHAPTER" "APPENDIX" 
	"GLOSSARY" "BIBLIOGRAPHY" "INDEX"
	"REFERENCE" "REFENTRY"
	"SECT1" 
	"BOOK" "SET" ;; just in case nothing else matches...
	))

(define component-level-element-list
  (list "PREFACE" "CHAPTER" "APPENDIX" 
	"GLOSSARY" "BIBLIOGRAPHY" "INDEX"
	"REFERENCE" "REFENTRY"
	"BOOK" "SET" ;; just in case nothing else matches...
	))

(define (ancestor-in-list nd gilist)
  (if (member (gi nd) gilist)
      nd
      (ancestor-in-list (parent nd) gilist)))

(define (chunk-level-parent #!optional (nd (current-node)))
  (ancestor-in-list nd chunk-level-element-list))

(define (component-level-parent nd)
  (ancestor-in-list nd component-level-element-list))

(define (prev-chunk-element #!optional (nd (current-node)))
  (let ((ndgi (gi nd))
	(iprec (ipreced nd))
	(piprec (ipreced (parent nd))))
    (cond ((equal? ndgi "SECT1")
	   (if (node-list-empty? (ipreced iprec)) ; this is the first sect1
	       (if (node-list-empty? (select-elements 
				      (descendants piprec) "SECT1"))
		   piprec
		   (node-list-last (select-elements 
				    (descendants piprec) "SECT1")))
	       iprec))
	  ((equal? ndgi "REFENTRY")
	   (if (node-list-empty? iprec)
	       piprec
	       iprec))
	  (else
	   (if (node-list-empty? (select-elements (descendants iprec) "SECT1"))
	       (if (and (not (node-list-empty? iprec))
			(or (equal? (gi iprec) "PREFACE")
			    (equal? (gi iprec) "CHAPTER")
			    (equal? (gi iprec) "REFERENCE")
			    (equal? (gi iprec) "APPENDIX")
			    (equal? (gi iprec) "GLOSSARY")
			    (equal? (gi iprec) "BIBLIOGRAPHY")
			    (equal? (gi iprec) "INDEX")))
		   iprec
		   (empty-node-list))
	       (node-list-last (select-elements (descendants iprec) "SECT1")))))))

(define (next-chunk-element #!optional (nd (current-node)))
  (let ((ndgi (gi nd))
	(inext (ifollow nd))
	(pinext (ifollow (parent nd))))
    (cond ((or (equal? ndgi "SECT1") (equal? ndgi "REFENTRY"))
	   (if (node-list-empty? inext)
	       pinext
	       inext))
	  ((equal? ndgi "REFERENCE")
	   (node-list-first
	    (select-elements (descendants nd) "REFENTRY")))
	  (else 
	   (if (node-list-empty? 
		(node-list-rest 
		 (select-elements (descendants nd) "SECT1")))
	       ;; there are none, or only 1 sect 1
	       inext
	       (node-list-first 
		(node-list-rest 
		 (select-elements (descendants nd) "SECT1"))))))))

(define (chunk-element-title #!optional (nd (current-node)))
  (if (equal? (gi nd) "REFENTRY")
      (let* ((refmeta (node-list-first 
		       (select-elements 
			(descendants nd) "REFMETA")))
	     (refentrytitle (node-list-first
			     (select-elements 
			      (descendants refmeta) "REFENTRYTITLE")))
	     (refmiscinfo (node-list-first
			   (select-elements 
			    (descendants refmeta) "REFMISCINFO"))))
	(string-append (data refmiscinfo) " " (data refentrytitle)))
      (let ((title (node-list-first 
		    (select-elements (descendants nd) "TITLE"))))
	(data title))))

(define (href-to target)
  ;; Return the HTML HREF for the given node.  For RefEntrys, this is
  ;; just the name of the file, for anything else it's the name of the file
  ;; with the fragment identifier for the specified id.
  (let* ((id (if (attribute-string "ID" target)
		 (attribute-string "ID" target)
		 (generate-xptr target)))
	 (entfile (html-file target))
	 (fragid (if (member (gi target) chunk-level-element-list)
		     ""
		     (string-append "#" id))))
    (string-append entfile fragid)))
	
(define (link-target idstring)
  ;; Return the HTML HREF for the given idstring.  For RefEntrys, this is
  ;; just the name of the file, for anything else it's the name of the file
  ;; with the fragment identifier for the specified id.
  (href-to (element-with-id idstring)))

;; ----------------------------------------------------------------------

(define (name-of-it elemnode)
  (let ((num (child-number elemnode)))
    (cond ((equal? (gi elemnode) "CHAPTER") 
	   (string-append "Chapter " (format-number (element-number) "1")))
	  ((equal? (gi elemnode) "APPENDIX") 
	   (string-append "Chapter " (format-number (element-number) "A")))
	  (else (chunk-element-title elemnode)))))

(define (header-navigation elemnode)
  (let ((prev (prev-chunk-element elemnode))
	(next (next-chunk-element elemnode)))
    (make element gi: "TABLE"
	  attributes: (list
		       (list "WIDTH" "515")
		       (list "BORDER" "0")
		       (list "CELLPADDING" "0")
		       (list "CELLSPACING" "0"))
	  (make element gi: "TR"
		(make element gi: "TH"
		      attributes: (list
				   (list "COLSPAN" "3")
				   (list "ALIGN" "center"))
		      ($nav-banner$)))
	  (make element gi: "TR"
		(make element gi: "TD"
		      attributes: (list
				   (list "WIDTH" "172")
				   (list "ALIGN" "left")
				   (list "VALIGN" "top"))
		      (if (node-list-empty? prev)
			  (make entity-ref name: "nbsp")
			  (make element gi: "A"
				attributes: (list
					     (list "HREF" (html-file prev)))
				(make sequence
				  (literal "Prev")))))
		(make element gi: "TD"
		      attributes: (list
				   (list "WIDTH" "171")
				   (list "ALIGN" "center")
				   (list "VALIGN" "top"))
		      (make sequence
			(literal (name-of-it (component-level-parent elemnode)))))
		(make element gi: "TD"
		      attributes: (list
				   (list "WIDTH" "172")
				   (list "ALIGN" "right")
				   (list "VALIGN" "top"))
		      (if (node-list-empty? next)
			  (make entity-ref name: "nbsp")
			  (make element gi: "A"
				attributes: (list
					     (list "HREF" (html-file 
							   (node-list-first 
							    next))))
				(make sequence
				  (literal "Next")))))))))

(define (footer-navigation elemnode)
  (let ((prev (prev-chunk-element elemnode))
	(next (next-chunk-element elemnode)))
    (make element gi: "TABLE"
	  attributes: (list
		       (list "WIDTH" "515")
		       (list "BORDER" "0")
		       (list "CELLPADDING" "0")
		       (list "CELLSPACING" "0"))
	  (make element gi: "TR"
		(make element gi: "TD"
		      attributes: (list
				   (list "WIDTH" "172")
				   (list "ALIGN" "left")
				   (list "VALIGN" "top"))
		      (if (node-list-empty? prev)
			  (make entity-ref name: "nbsp")
			  (make element gi: "A"
				attributes: (list
					     (list "HREF" (html-file 
							    prev)))
				(make sequence
				  (literal "Prev")))))
		(make element gi: "TD"
		      attributes: (list
				   (list "WIDTH" "171")
				   (list "ALIGN" "center")
				   (list "VALIGN" "top"))
		      ($nav-home$))
		(make element gi: "TD"
		      attributes: (list
				   (list "WIDTH" "172")
				   (list "ALIGN" "right")
				   (list "VALIGN" "top"))
		      (if (node-list-empty? next)
			  (make entity-ref name: "nbsp")
			  (make element gi: "A"
				attributes: (list
					     (list "HREF" (html-file 
							    next)))
				(make sequence
				  (literal "Next"))))))

	  (make element gi: "TR"
		(make element gi: "TD"
		      attributes: (list
				   (list "WIDTH" "172")
				   (list "ALIGN" "left")
				   (list "VALIGN" "top"))
		      (if (node-list-empty? prev)
			  (make entity-ref name: "nbsp")
			  (make sequence
			    (literal (chunk-element-title prev)))))
		(make element gi: "TD"
		      attributes: (list
				   (list "WIDTH" "171")
				   (list "ALIGN" "center")
				   (list "VALIGN" "top"))
		      ($nav-index$))
		(make element gi: "TD"
		      attributes: (list
				   (list "WIDTH" "172")
				   (list "ALIGN" "right")
				   (list "VALIGN" "top"))
		      (if (node-list-empty? next)
			  (make entity-ref name: "nbsp")
			  (make sequence
			    (literal (chunk-element-title next)))))))))

